# -*- coding: utf-8 -*- 
# @Time     : 2019-11-01 10:02
# @Author   : binger

from .html_to_image import html2image
from jinja2 import Template
from . import FontAttr, FontDraw
import os


class Color(object):
    def __init__(self, color):
        """
        支持：数组形式的 rgba 与 rgb 表示，和 "#000000" 表示
        :param color:
        """
        self.color = color

    def __str__(self):
        if isinstance(self.color, (tuple, list)):
            if len(self.color) == 3:
                return "rgb{}".format(repr(tuple(self.color)))
            else:
                return "rgba{}".format(repr(tuple(self.color)))
        elif isinstance(self.color, Color):
            obj = self.color
            self.color = obj.color
            return obj.__str__()
        else:
            return self.color


class EffectFont(object):
    def __init__(self):
        self.base = None
        self.shadow_extend_offset = None

        self.stroke_enable = False  # 描边功能渲染使用启用
        self.stroke_width = 1  # 描边画笔的宽
        self.stroke_color = None  # 描边颜色

        self.shadow_enable = False  # 阴影功能开启
        self.shadow_x = self.shadow_y = 0  # 阴影的偏移量
        self.shadow_sigma = 0.0  # 阴影的模糊度
        self.shadow_color = ""  # 阴影的颜色

        self.gradient_enable = False  # 渐变功能是否启动
        self._fill_color = ""  # 字体前景色填充色
        self.fill_type = "color"

        self.text_content = None  # 文字内容（换行时是 <br/>）
        self.text_shadow = None  # 阴影时文字内容（换行时<A>）
        self._text = None
        self.limit_width = None  # 达到宽度自动换行
        self.is_auto_wrap = False  # 是否开启自动宽度限制换行
        self.weight = 'normal'

    def set_text_base(self, size, path, line_height=None, line_spacing_par=0.1, char_spacing=None, char_spacing_par=0.1,
                      variable_spacing=True, limit_height=None, limit_width=None, limit_count=None, align="left",
                      limit_line=None, line_sep='\n', fill_color=None, weight="normal"):

        _vars = vars().copy()
        _vars.pop("self")
        weight = _vars.pop('weight')
        _vars["path"] = os.path.abspath(_vars["path"])
        self.base = FontAttr(**_vars)
        self.weight = weight
        return True

    def shadow(self, color, sigma=0.0, x=0, y=0):
        self.shadow_enable = True
        self.shadow_x = x
        self.shadow_y = y
        self.shadow_sigma = sigma
        self.shadow_color = color
        self.shadow_extend_offset = (x, y)
        return True

    def gradient(self, color_pos_list, angle, type="linear"):
        """

        :param color_pos_list: [("#da7eeb", 0), ("#9070d8", 0.99)]
        :param type:  "linear" or "radial"
        :param angle:  角度90或者偏移量 1px, 2px 或者 center
        :return:
        """
        if type == "linear":
            angle = "{angle}deg".format(angle=angle)
        else:

            angle = "{}px, {}px".format(*angle[:2]) if isinstance(angle, (
                tuple, list)) else angle
            angle = "circle farthest-corner at {}".format(angle)
        color_change = ", ".join(["{} {}%".format(resp[0], 100 * resp[1]) for resp in color_pos_list])
        content = """{type}-gradient({angle} , {color_change});
        """.format(
            type=type, angle=angle, color_change=color_change
        )
        self._fill_color = content
        self.gradient_enable = True
        return True

    @property
    def fill_color(self):
        return self._fill_color

    @fill_color.setter
    def fill_color(self, color):
        assert isinstance(color, tuple)
        self.base.fill_color = color
        self._fill_color = Color(color)
        self.gradient_enable = False
        return True

    def stroke(self, color, width):
        self.stroke_enable = True
        self.stroke_width = width
        self.stroke_color = color
        return True

    @property
    def text(self):
        return self._text

    @text.setter
    def text(self, text):
        self._text = text
        txt_list = self._text.split("\n")
        self.text_content = "<br/>".join(txt_list)
        self.text_shadow = "\A".join(txt_list)


class FontDrawByHtml(object):
    def __init__(self):
        self._html = None
        self._request_size = None
        self._extend_offset = None  # 阴影引起的区域多偏移
        self.selector_element = "a"

        self._tpl = Template(self.init_template())

    @property
    def html(self):
        return self._html

    @classmethod
    def init_template(cls):
        return """
<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
</head>
<style>
    body {
        margin: 0;
        position: relative;
    }

    @font-face {
        font-family: custom_font;
        src: url("{{ font.base.path }}");
    }

    #text {
        position: absolute;
        left: 0;
        top: 0;
        font-size: {{ font.base.size }}px;
        /**
            字号
        */
        line-height: {{font.base.line_height}}px;
        letter-spacing: {{font.base.char_spacing}}px;
        text-align: {{font.base.align}};
        font-family: custom_font;
        /**
            字体
        */
        font-weight: {{ font.weight }};
        /**
            当没有渐变时 不用background-image,把颜色填在 -webkit-text-fill-color:;
            当有渐变时 设置 -webkit-text-fill-color:transparent;
        */
        {%- if font.gradient_enable -%}
            -webkit-text-fill-color: transparent;

            /*-webkit-text-fill-color: #7b759a;*/
            /**
                不用渐变时的颜色
            */
            background-image: {{ font.fill_color}};
            /**
                radial-gradient：中心渐变（线性渐变会给另一个background-image）
                circle: 渐变形状为圆形（和设计师确认过不会换）;
                farthest-corner: 以离中心点最远的角的距离为 100% （和设计师确认过不会换）;
                center: 渐变中心 可以替换为偏移量 1px 1px（x轴偏移量 y轴偏移量） （和设计师讨论过可能支持设置9个点）;
                #da7eeb 0%: 颜色和百分比  在 0%（中心点） 位置的颜色为 #da7eeb;
                #9070d8 99%： 颜色和百分比 在 99%（中心点距离最远角的百分之99的地方） 位置的颜色为 #9070d8;
                颜色和百分比可以添加多对；
            */
            /*background-image: linear-gradient(0deg , #bd8749 0%, #efdfac 69%);*/
            /**
                linear-gradient: 线性渐变;
                90deg: 渐变角度;
                #f6d365 0%: 颜色和百分比 可以多对
            */
        {%- else -%}
            -webkit-text-fill-color: {{font.fill_color}};
        {%- endif -%}

        {%- if font.stroke_enable -%}
            -webkit-text-stroke: {{font.stroke_width}}px {{font.stroke_color}};
            /**
                内描边Ï
                1px: 描边1像素宽;
                #4e1269: 描边的颜色;
            */
        {%- endif -%}
        -webkit-background-clip: text;
        position: absolute;
        left: 0;
        top: 0;
        {%- if font.base.limit_width -%}
            limit_width: {{ font.base.limit_width }}px;
            word-break: bread-all;
        {%- endif -%}
        white-space: pre-wrap;
    }
    {%- if font.shadow_enable -%}
    #text::before {

        text-shadow: {{font.shadow_x}}px {{font.shadow_y}}px {{font.shadow_sigma}}px {{font.shadow_color}};
        /**
            阴影
            0px:x轴偏移量;
            10px: y轴偏移量;
            12px: 模糊;
            rgba(80, 31, 191, 0.3): rgba(前三个数值表示颜色,透明度);
        */
        color: transparent;
        display: inline;
        content: "{{ font.text_shadow }}";
        position: absolute;
        left: 0;
        top: 0;
        line-height: {{font.base.line_height}}px;
        letter-spacing: {{font.base.char_spacing}}px;
        text-align: {{font.base.align}};
        z-index: -1;
        {%- if font.base.limit_width -%}
            limit_width: {{ font.base.limit_width }}px;
            word-break: bread-all;
        {%- endif -%}
        white-space: pre-wrap;
    }
    {%- endif -%}
</style>
<body>
<a id="text">{{ font.text_content }}</a>
</body>
</html> 
        """

    def save(self, filename):
        if self._html:
            with open(filename, "w") as fw:
                fw.write(self._html)

    def to_image(self):
        from tempfile import NamedTemporaryFile

        with NamedTemporaryFile(mode="w", suffix=".html") as fw:
            fw.write(self._html)
            fw.seek(0)
            img = html2image("file://{}".format(fw.name), size=self._request_size,
                             extend_offset=self._extend_offset,
                             selector_element=self.selector_element)
            return img

    def build(self, font):
        assert isinstance(font, EffectFont)
        msg = self._html = self._tpl.render(font=font)

        if font.shadow_enable:
            self._extend_offset = font.shadow_extend_offset
        return msg

    @classmethod
    def render_to_image(cls, font, request_area_size=(800, 800)):
        obj = cls()
        obj._request_size = request_area_size
        obj.build(font)

        return obj.to_image()



# -*- coding: utf-8 -*- 
# @Time     : 2019-10-09 14:42
# @Author   : binger

from pyppeteer import launch
import asyncio
from PIL import Image
from io import BytesIO
import os

__advocate_loop = asyncio.new_event_loop()

executablePath = "chromium/chrome"
options = dict(handleSIGINT=False,
               handleSIGTERM=False,
               handleSIGHUP=False,
               autoClose=False,
               args=['--no-sandbox', '--disable-setuid-sandbox'])
if os.path.exists(executablePath):
    options["executablePath"] = executablePath


async def to_buffer(page_path, size, extend_offset=None, selector_element=None):
    browser = await launch(options)
    page = await browser.newPage()
    try:
        await page.setViewport(viewport={"width": size[0], "height": size[1]})
        # await page.setJavaScriptEnabled(enabled=False)
        await page.goto(page_path)
        # await page.screenshot({'path': _OUTFILE, 'fullPage': True, 'omitBackground': "transparency"})
        element = selector_element and await page.querySelector(selector_element)

        if element:
            box = await element.boundingBox()
            if extend_offset:
                box["width"] += extend_offset[0]
                box["height"] += extend_offset[1]
            buffer = await element.screenshot(
                {'type': 'png', 'omitBackground': "transparency", "clip": box})
            # {'type': 'png', 'fullPage': True, 'omitBackground': "transparency", "clip": box})
        else:
            buffer = await page.screenshot({'type': 'png', 'fullPage': True, 'omitBackground': "transparency"})
    finally:
        await page.close()
    return buffer


async def to_image(page_path, size, extend_offset=None, selector_element=None):
    buffer = await to_buffer(page_path, size, extend_offset, selector_element)
    image = Image.open(BytesIO(buffer))
    return image


import threading


def mutex_self_lock(cls):
    def decorator(*args, **kwargs):
        if decorator.mutex.acquire():
            result = None
            try:
                result = cls(*args, **kwargs)
            except:
                pass
            finally:
                decorator.mutex.release()
                return result

    decorator.mutex = threading.Lock()
    return decorator


# @mutex_self_lock
def html2image(page_path, size, extend_offset=None, selector_element=None):
    asyncio.set_event_loop(__advocate_loop)
    return __advocate_loop.run_until_complete(to_image(page_path, size, extend_offset, selector_element))



# -*- coding: utf-8 -*- 
# @Time     : 2019-11-08 10:08
# @Author   : binger

from .draw_by_html import EffectFont
from .draw_by_html import FontDrawByHtml
from . import FontDraw, FontAttr
from PIL import Image, ImageDraw, ImageFont


class FontFactory(object):
    def __init__(self, effect_font, render_by_mixed=False):
        """
        :param render_by_mixed: False 只有浏览器生成，
        :param fill_type:
        """
        assert isinstance(effect_font, EffectFont)
        self.render_by_mixed = render_by_mixed
        self._effect_font = effect_font

    def is_effected(self):
        return any([self._effect_font.gradient_enable, self._effect_font.shadow_enable, self._effect_font.stroke_enable,
                    self._effect_font.weight != 'normal'])

    def get_line_size(self, size, text):
        return FontDraw(self._effect_font.base).get_line_size(text, size=size)

    def get_max_canvas_size(self, text):
        return FontDraw(self._effect_font.base).max_canvas_size(text, is_truncated=True)

    def get_text_size_at_width(self, size, text, width, spacing=4):
        effect_font = ImageFont.truetype(font=self._effect_font.base.path, size=size)
        len_text = len(text)

        line = text
        remain = ''

        temp_width = 0
        for n in range(len_text):
            temp_width += effect_font.getsize(text[n])[0]
            if temp_width > width:
                line = text[:n - 1]
                remain = text[n - 1:]
                break
            temp_width += spacing

        return line, remain

    def render_to_rng(self):
        if self.render_by_mixed and not self.is_effected():
            limit_text_cb = lambda *args, **kwargs: True
            _app = FontDraw(self._effect_font.base)
            resp = _app.write(self._effect_font.text, align=self._effect_font.base.align, limit_text_cb=limit_text_cb)
            img = resp.img
        else:
            img = FontDrawByHtml.render_to_image(self._effect_font)
        return img


if __name__ == '__main__':
    font = EffectFont()
    path = '../simkai.ttf'
    import os
    print(os.path.abspath(path))
    font.set_text_base(size=100, path=path)
    font.base.clear_margin=True
    # font.shadow((80, 31, 191, 0.3), sigma=8, x=0, y=6)
    # font.gradient([("#da7eeb", 0), ("#9070d8", 0.5)], angle="center", type="radial")
    # font.fill_color = "#da7eeb"
    font.fill_color = (1, 1, 1, 255)
    # font.stroke("#4e1269", 1)
    font.text = '实例\n中国\n实例'
    obj = FontFactory(font, render_by_mixed=True)
    img = obj.render_to_rng()
    print(img.size)
    img.save("11.png")



# -*- coding: utf-8 -*-
# @Time     : 2019-12-04 16:21
# @Author   : binger

__all__ = ["FontDraw", "FontAttr", "FontDrawResult"]

from PIL import Image, ImageDraw, ImageFont
from operator import itemgetter
from collections import namedtuple
import math


# import attr


class FontAttr(object):

    def __init__(self, size, path, line_height=None, line_spacing_par=0.1, char_spacing=None, char_spacing_par=0.1,
                 variable_spacing=True, limit_height=None, limit_width=None, limit_count=None, align="left",
                 limit_line=None, line_sep='\n', fill_color=None, clear_margin=False):
        self.path = path
        self._size = size

        # 三个一起使用，配合字体大小变化重新计算 line_height 和 char_spacing
        self.variable_spacing = variable_spacing
        self._line_height = line_height
        self._char_spacing = char_spacing

        self.char_spacing_par = char_spacing_par
        self.line_spacing_par = line_spacing_par
        self.line_height = line_height or int(size * (1 + self.line_spacing_par))
        self.char_spacing = char_spacing or int(size * self.char_spacing_par)
        self.align = align
        self.line_sep = line_sep
        self.limit_width = limit_width
        self.limit_height = limit_height
        self.limit_count = limit_count and int(limit_count)
        self.limit_line = limit_line and int(limit_line)
        # limit_count, limit_width 达到限制后处理，执行处理
        # limit_count, limit_line 默认取值从1开始
        self.fill_color = fill_color and tuple(fill_color)  # RGBA
        self.clear_margin = clear_margin  # 默认从上，左开始血，即清理的下和右的边距
        self._font = None

    @property
    def size(self):
        return self._size

    @size.setter
    def size(self, size):
        self._size = size
        if self.variable_spacing:
            self.line_height = self._line_height or int(size * (1 + self.line_spacing_par))
            self.char_spacing = self._char_spacing or int(size * self.char_spacing_par)

    @property
    def font(self):
        if not self._font:
            self._font = ImageFont.truetype(self.path, size=self.size)
        return self._font

    def reset_font(self):
        """重新产生pillow字体模型（自已库与大小的绑定）"""
        self._font = ImageFont.truetype(self.path, size=self.size)


FontDrawResult = namedtuple('FontDrawResult', ['offset', 'start', 'lines', 'img', 'h_margin'])


class FontDraw(object):
    def __init__(self, font, bg=None):
        self._font = font
        self._bg = bg
        self.is_write_on_bg = bool(bg)
        self._draw = bg and ImageDraw.Draw(bg)

    def __init_draw(self, text, is_truncated=False):
        size = self.max_canvas_size(text, is_truncated)
        color = self._font.fill_color
        self._bg = bg = Image.new('RGBA', size, color=(*color[:3], 0))
        self._draw = ImageDraw.Draw(bg)

    def max_canvas_size(self, text, is_truncated=False):
        """根据文字，近似计算画布大小"""
        lines = text.strip().split(self._font.line_sep)
        width_list, height_list = [], []
        for line in lines:

            if len(line.strip()) == 0:
                continue

            if is_truncated:
                w, h = None, None
                w_max, w_min = None, None
                if self._font.limit_count:
                    w_list, h_list = [], []
                    for i in range(0, len(line), self._font.limit_count):
                        _w, _h = self.get_line_size(line[i: i + self._font.limit_count])
                        w_list.append(_w)
                        h_list.append(_h)
                    w_max = max(w_list) if w_list else 0
                    w_min = min(w_list) if w_list else 0
                    h = len(h_list) * self._font.line_height
                if self._font.limit_width:
                    _w, _h = self.get_line_size(line)
                    w2 = self._font.limit_width
                    h_max = math.ceil(_w / (w_min or w2)) * self._font.line_height
                    if h:
                        h = max([h, h_max])
                    else:
                        h = h_max

                    if w_max:
                        w = max([w_max, w2])
                    else:
                        w = w2
                else:
                    w, h = self.get_line_size(line)
                    h = self._font.line_height
            else:
                w, h = self.get_line_size(line)
                h = self._font.line_height
            width_list.append(w)
            height_list.append(h)
        return math.ceil(max(width_list) if width_list else 0), math.ceil(sum(height_list))

    @staticmethod
    def get_char_size(c, path, size):
        """
        获得一个字的宽和高
        :param c:
        :param path:
        :param size:
        :return: (limit_width, height)
        """
        assert len(c) == 1
        return ImageFont.truetype(font=path, size=size).getsize(c)

    def get_line_size(self, line, size=None, char_spacing=None):
        """
        获取一行的宽度
        :param line:
        :param size:
        :param char_spacing:
        :return: (limit_width, height)
        """
        char_spacing = char_spacing or self._font.char_spacing
        font_obj = ImageFont.truetype(font=self._font.path, size=size or self._font.size)
        x_list = []
        y_list = []
        for i, c in enumerate(line):
            _size = font_obj.getsize(c)
            x_list.append(_size[0])
            y_list.append(_size[1])
        x = sum(x_list) + (len(line) - 1) * char_spacing
        y = max(y_list) if y_list else 0
        return x, y

    def write_line(self, line, position=None, align="left", limit_text_cb=None):
        """

        :param line:
        :param position:
        :param align:
        :param limit_text_cb: progress(index: 序列号，line: 写入行内容， state: 引起判断的条件，字数或者长度限制)，
        返回为True代表换行，False代表丢弃，limit_text_cb
        :return: (x方法的偏移量，y方向的偏移量), 写的个数，剩余的待输入的内容
        """
        res = ""  # 作为判断时候换行使用
        x, y = position = position or (0, 0)
        h_list = []
        align = 1 if align == "left" else -1
        count = 0
        for i, c in enumerate(line):
            if i != 0:
                x += self._font.char_spacing * align
            c_w, c_h = self.get_char_size(c=c, path=self._font.path, size=self._font.size)

            if limit_text_cb:
                # 为什么不定义限制，默认自动换行，limit_text_cb 处理，如果默认为None，则会出现换行或者截断误会抓取位置
                if self._font.limit_count and i >= self._font.limit_count - 1:
                    # 按每行文字个数执行换行保留, 还是截断丢弃
                    if limit_text_cb(i, line, 'limit_count'):
                        res = line[i:]
                    else:
                        res = ''
                    break
                if self._font.limit_width and x + c_w * align > self._font.limit_width:
                    # 按达到预定宽度，判断是换行，还是丢弃剩余的
                    if limit_text_cb(i, line, 'limit_width'):
                        res = line[i:]
                    else:
                        res = ''
                    break

            self._draw.text(xy=(x, y), text=c, font=self._font.font, fill=self._font.fill_color)
            x += c_w * align
            h_list.append(c_h)
            count = i + 1

        return (x - position[0], max(h_list) if h_list else 0), count, res

    def write(self, text, position=None, align='left', limit_text_cb=None):
        """
                有宽度限制输入文字，达到限制，换行或者丢失
        :param text:
        :param position:
        :param align:
        :param limit_text_cb: 1. 函数没定义，不做处理 2. 返回值为True，执行保留换行， 2. False 执行截断丢弃
        :return:
        """
        self._font.reset_font()
        self._draw or self.__init_draw(text, is_truncated=bool(limit_text_cb))
        x, y = position = position or (0, 0)

        lines = text.strip().split(self._font.line_sep)

        line_width = []
        n = 0
        new_lines = []
        margin_h = [0, 0]
        for line in lines:
            while line:
                if len(line.strip()) == 0:
                    continue

                # offset, count, rest_line = self.write_line(line, position=(x, y + 1), align=align,
                offset, count, rest_line = self.write_line(line, position=(x, y), align=align,
                                                           limit_text_cb=limit_text_cb)
                new_lines.append(line[:count])
                line = rest_line

                # 不加,会出现字体不自然
                line_width.append(offset[0])
                if n == 0:
                    margin_h[0] = math.ceil((self._font.line_height - offset[1]) / 2)
                margin_h[1] = int(self._font.line_height - offset[1])

                y += self._font.line_height
                n += 1
                # 行数限制：如果换行次多余规定的次数，则忽略后面的
                if self._font.limit_line and n >= self._font.limit_line:
                    break

        offset = [max(line_width) if line_width else 0, y - position[1]]
        if not self.is_write_on_bg:
            all_h = offset[1]
            if self._font.clear_margin:
                offset[1] = all_h - margin_h[1]
                margin_h[1] -= margin_h[0]
            else:
                margin_h = [0, 0]
            self._bg = self._crop(offset, start=position)

        result = FontDrawResult(
            offset=tuple(offset),
            start=position,
            lines=new_lines,
            img=self._bg,
            h_margin=margin_h
        )
        return result

    def get_size_at_limit_range(self, text, size, char_spacing_par=0.1, min_size=12):
        """在运行字体变小时调用, 通过限制获取最大的字体"""
        lines = text.strip().split(self._font.line_sep)
        if not lines:
            return
        _char_spacing = int(size * char_spacing_par)
        handle = lambda i, line: [i, *self.get_line_size(line, size, char_spacing=_char_spacing)]
        size_iter = list(map(lambda msg: handle(*msg), enumerate(lines)))
        w_max = max(size_iter, key=itemgetter(1))
        h_max = max(size_iter, key=itemgetter(2))
        h_max_length = int(h_max[2])
        h_max_index = h_max[0]
        w_max_lenght = int(w_max[1])
        w_max_index = w_max[0]

        _temp_new_size = None
        result_size = size
        n = len(lines)
        if self._font.limit_height:
            is_first = True
            while True:
                if n == 1 and (h_max_length > self._font.limit_height):
                    if is_first:
                        _temp_new_size = self._font.limit_height
                else:

                    # limit_size = math.ceil(
                    #     (self._font.limit_height - (n - 1) * self._font.line_height) / n)

                    limit_size = math.ceil(
                        self._font.limit_height / (n + (n - 1) * self._font.line_spacing_par)
                    )
                    if h_max_length > limit_size:
                        if is_first:
                            _temp_new_size = limit_size
                    else:
                        break

                # 字体下限
                if _temp_new_size == min_size:
                    return min_size

                _temp_new_size -= 1
                h_max_length = \
                    self.get_line_size(lines[h_max_index], _temp_new_size, int(_temp_new_size * char_spacing_par))[1]
        if _temp_new_size:
            result_size = _temp_new_size
            w_max_lenght = \
                self.get_line_size(lines[w_max_index], _temp_new_size, int(_temp_new_size * char_spacing_par))[0]

        if self._font.limit_width:
            line = lines[w_max_index]
            if w_max_lenght > self._font.limit_width:
                _temp_new_size = math.ceil(self._font.limit_width * result_size / w_max_lenght)
                while True:
                    if self.get_line_size(line, _temp_new_size, int(_temp_new_size * char_spacing_par))[
                        0] < self._font.limit_width:
                        result_size = _temp_new_size
                        break
                    else:
                        # 字体下限
                        if _temp_new_size == min_size:
                            return min_size

                        _temp_new_size -= 1
        return result_size

    def _crop(self, offset, start=None):
        start = start or (0, 0)
        return self._bg.crop((start[0], start[1], start[0] + offset[0], start[1] + offset[1]))


if __name__ == '__main__':
    # -*- coding: utf-8 -*-
    from pyfont import FontAttr, FontDraw
    from PIL import Image
    import os

    image = Image.new('RGBA', (int(400), int(400)), (1, 1, 1, 0))
    path = '../simkai.ttf'
    path = os.path.abspath(path)
    print(path)
    font = FontAttr(path=path, size=20, limit_width=150, fill_color=(1, 1, 1, 255))
    # obj = FontDraw(bg=image, font=font)
    obj = FontDraw(font=font)


    # 通过控制 limit_width，limit_count 与 传入回调返回，决定是否换行，或者丢弃
    # limit_text_cb=None 不处理（超过）
    # limit_text_cb 返回值False丢弃多余，返回True（保留换行）
    def limit_text_cb(index, line, state):
        print('index:', index, line[index:], state)
        return False


    text = "我们是中国人，我爱我的祖国\n你好"
    size = obj.get_size_at_limit_range(text, font.size)
    print("size: ", size)
    # font.size = 10
    result = obj.write(text=text, limit_text_cb=limit_text_cb)
    img = result.img
    print(img.size)
    img.save("font.png")
    print(font.size, font.line_height)



# -*- coding: utf-8 -*- 
# @Time     : 2020-02-11 23:32
# @Author   : binger


name = "pyfont"
version_info = (0, 0, 2, 2002618)
__version__ = ".".join([str(v) for v in version_info])
__description__ = '生成文字图片'

from .font import FontAttr, FontDraw, FontDrawResult
from .api import EffectFont, FontFactory



if __name__ == '__main__':
    from pyfont import EffectFont, FontFactory

    font = EffectFont()
    path = '../simkai.ttf'
    import os

    print(os.path.abspath(path))
    font.set_text_base(size=100, path=path)
    font.base.clear_margin = True
    # font.shadow((80, 31, 191, 0.3), sigma=8, x=0, y=6)
    # font.gradient([("#da7eeb", 0), ("#9070d8", 0.5)], angle="center", type="radial")
    # font.fill_color = "#da7eeb"
    font.fill_color = (1, 1, 1, 255)
    # font.stroke("#4e1269", 1)
    font.text = '实例\n中国\n实例'
    obj = FontFactory(font, render_by_mixed=True)
    img = obj.render_to_rng()
    print(img.size)
    img.save("11.png")



import os
from setuptools import setup, find_packages
from pyfont import name, __description__, __version__

# read dev requirements
fname = os.path.join(os.path.dirname(__file__), 'requirements.txt')
with open(fname) as f:
    install_requires = [l.strip() for l in f.readlines()]
root = os.path.basename(os.path.dirname(os.path.abspath(__file__)))

with open("MAINTAINERS") as f:
    lines = [l.strip() for l in f.readlines()]
    author, author_email = lines[0].split(sep=" ")
    if len(lines) > 1:
        maintainer, maintainer_email = lines[1].split(sep=" ")
    else:
        maintainer, maintainer_email = author, author_email

setup(
    name=name,
    version=__version__,
    packages=find_packages(exclude=['examples', 'tests']),
    url='https://github.com/BingerYang/{}'.format(root),
    license='MITT',
    author=author,
    author_email=author_email,
    maintainer=maintainer,
    maintainer_email=maintainer_email,
    description=__description__,
    long_description=open('README.rst', encoding='utf-8').read(),
    long_description_content_type="text/x-rst",

    python_requires='>=3.4',
    platforms=['all'],
    include_package_data=True,
)
